import sys
import time

from ardi.driver import drivercore,driverbase

from suds.client import Client
from suds.bindings import binding

class ardipiwebsdriver:
    def __init__(self):
        self.connected = False
        self.polling = True        
        self.subscription = ''
        self.ticket = ""
        self.optimised = False
        self.prefix = ""

    def SetAddress(self, addr):
        self.connected = False
        bits = addr.split("^")
        self.address = bits[0]
        if (len(bits) > 1):
            self.prefix = bits[1]
    
    def Connect(self):
        try:		
            #Get WSDL file for PI Web Services
            #print 'Checking ' + self.address + " - " + self.address[4:7]
            if (self.address[4:7] == "://"):
                self.address = self.address[7:]
            
            url = 'http://' + self.address + '/PIWebServices/PITimeSeries.svc?wsdl'
            self.core.logger.info("Loading " + url)
            
            self.core.logger.info("Connecting via SOAP 1.1")
            
            try:
                self.client = Client(url)
                #    self.client = Client(url, headers = {'Content-Type' : 'application/soap+xml' })
            except Exception as e:
                self.core.logger.error('Failure During Connect: ' + str(e))
                #self.core.logger.exception('Exception During Connect')
                return
            
            try:
                ver = self.client.service.GetProductVersion()
            except Exception as e:
                self.core.logger.error('Failure During GetProductVersion: ' + str(e))
                return
            
            self.core.logger.info("Connected to PI Web Services version " + ver)			

            self.connected = True        
        except:			
            return False
        return True

    def Optimise(self):
        self.optimised = False
        
        strset = self.client.factory.create('ArrayOfString')
        #print strset
        
        strset.string = []

        for itm in self.core.pointlist:
        	strset.string.append(self.prefix + itm.address)
        	        
        try:
            results = self.client.service.SignUpForPIUpdates(strset,10)
        except Exception as e:
            self.core.logger.info("Failed Subscribing To : " + str(strset))
            self.core.logger.error("Exception In Subscription : " + str(e))
            return

        if (results.Errors[0] != 0):            
            self.core.logger.info("Unable to Subscribe - PIWS Call Failed")
            try:
                self.core.logger.error("Web Services Error: " + results.ErrDescs[0])
            except:
                self.core.logger.error("Unknown Error Type (Exception While Getting Error Info)")
                
            self.ticket = ""
            self.Disconnect()
            time.sleep(10)
            return

        self.ticket = results.UpdateTicket					
        self.iswaiting = True
        self.optimised = True	

    def Disconnect(self):
        self.connected = False
        #self.client.close()
        
    def Poll(self):

		#guid = self.client.factory.create('ns0:guid')
		#filter = self.client.factory.create('UpdateFilterType')
		
		if (self.connected == False):
		    return
		
		if (self.optimised == False):
		    self.Optimise()
		    if (self.optimised == False):
		        return
		    
		values = self.client.service.GetPIUpdates(self.ticket,60,"SnapshotAndArchive")		
		
		indx = 0
		for itm in self.core.pointlist:
			val = values.TimeSeriesUpdates[indx].Updates.TimedValueUpdate[0].value						
			self.core.UpdateData(itm.code,str(val))			
			indx = indx + 1
		
		return
		try:
			values = self.opc.read(group='ardidaq')
		except:
			try:
				self.opc.remove('ardidaq')
			except:
				pass
			try:
				self.opc.close()
			except:
				pass
				
			self.connected = False
			return
			
		print values

		indx = 0
		for itm in self.core.pointlist:
			quality = values[indx][2]			
			if (quality == 'Good'):	
				val = values[indx][1]						
				self.core.UpdateData(itm.code,str(val))
			else:
				self.core.UpdateData(itm.code,'^')
			indx = indx + 1

class driverfactory:
    def createinstance(self):
        return ardipiwebsdriver()
            
if __name__ == "__main__":
    sdf = driverfactory()
    base = driverbase.ardidriver()
    base.start(sdf)